@@ -14,6 +14,7 @@ import sys
 import numpy as np
 import os
 from collections import defaultdict
+from shutil import copy2
 
 ## Parse Markdown
 from markdown_it import MarkdownIt # pip install markdown-it-py 
@@ -73,7 +74,7 @@ def split_body_parts(num_parts,body_parts):
         try:
             parts_dict[part]['answer']['title'] = MDRenderer().render(tokens[pa[0]+1:pa[1]], mdit.options, env)
         except IndexError:
-            print("Check the heading level of 'Answer Section', it should be ### Answer Section.")
+            print("Check the heading levels, is there one that doesn't belong? Or is the heading level incorrect? For e.g., it should be ### Answer Section (this is not necessarily where the issue is).")
             raise
 
         parts_dict[part]['content'] = MDRenderer().render(tokens[ptt[1]+1:pa[0]], mdit.options, env)
@@ -94,7 +95,7 @@ def read_md_problem(filepath):
             - `num_parts` - Number of parts in the problem (integer).
     """
 
-    mdtext = pathlib.Path(filepath).read_text(encoding='raw_unicode_escape')
+    mdtext = pathlib.Path(filepath).read_text()
 
     # Deal with YAML header
     header_text = mdtext.rsplit('---\n')[1]
@@ -353,20 +354,23 @@ def process_attribution(source):
     Returns:
         string (str): returns the html of the attribution
     """
+    try:
+        if 'openstax-physics-vol1' in source:
+            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 1](https://openstax.org/details/books/university-physics-volume-1) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
 
-    if 'openstax-physics-vol1' in source:
-        attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 1](https://openstax.org/details/books/university-physics-volume-1) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
+        elif 'openstax-physics-vol2' in source:
+            attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 2](https://openstax.org/details/books/university-physics-volume-2) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
 
-    elif 'openstax-physics-vol2' in source:
-        attribution_text = "![Image representing the Creative Commons 4.0 BY license.](https://i.creativecommons.org/l/by/4.0/88x31.png) Problem is from the [OpenStax University Physics Volume 2](https://openstax.org/details/books/university-physics-volume-2) textbook, licensed under the [CC-BY 4.0 license](https://creativecommons.org/licenses/by/4.0/)."
+        elif 'ubc-mech2' in source:
+            raise NotImplementedError
 
-    elif 'ubc-mech2' in source:
-        raise NotImplementedError
-
-    elif 'standard' in source:
-        attribution_text = "![Image representing the Creative Commons 4.0 BY-NC-SA license.](https://mirrors.creativecommons.org/presskit/buttons/88x31/png/by-nc-sa.png) Problem is licensed under the [CC-BY-NC-SA 4.0 license](https://creativecommons.org/licenses/by-nc-sa/4.0/)."
-
-    return attribution_text
+        elif 'standard' in source:
+            attribution_text = "![Image representing the Creative Commons 4.0 BY-NC-SA license.](https://mirrors.creativecommons.org/presskit/buttons/88x31/png/by-nc-sa.png) Problem is licensed under the [CC-BY-NC-SA 4.0 license](https://creativecommons.org/licenses/by-nc-sa/4.0/)."
+    
+        return attribution_text
+    
+    except TypeError:
+        print("You probably need to update the template, the 'attribution' key seems to be missing.")
 
 def process_question(input_file, output_path):
 
@@ -437,20 +441,24 @@ def process_question(input_file, output_path):
             else:
                 raise NotImplementedError(f"This question type ({q_type}) is not yet implemented.")
 
-
             question_html += "</div>\n</div>\n\n\n"
 
     # Add Attribution
     question_html += f"\n\n<markdown>---\n{process_attribution(parsed_q['header'].get('attribution'))}\n</markdown>"
 
     # Write question.html file
-    #(output_path / "question.html").write_text(question_html,encoding='raw_unicode_escape')
+    (output_path / "question.html").write_text(question_html) #,encoding='raw_unicode_escape')
 
     ### TODO solve the issue with the latex escape sequences, this is a workaround
-    with open((output_path / "question.html"), "w") as qfile:
-        print(f"{question_html}", file=qfile)
+    # with open((output_path / "question.html"), "w") as qfile:
+    #     print(f"{question_html}", file=qfile)
 
     # Write server.py file
     write_server_py(output_path,parsed_q)
 
-    # TODO: Find and move any image assets 
+    # Move image assets
+    files_to_copy = parsed_q['header'].get('assets')
+    pl_path =  output_path / "clientFilesQuestion"
+    pl_path.parent.mkdir(parents=True, exist_ok=True)
+    if files_to_copy:
+        [copy2(pathlib.Path(source_filepath).parent / fl, pl_path) for fl in files_to_copy]
\ No newline at end of file
